// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a GPL v3 license that can be
// found in the LICENSE file.

import 'dart:async';

import 'package:flutter/services.dart';

import 'app.dart';

class ApkAdmin {
  static const _channel = MethodChannel('com.heda.apk_admin');

  /// This method gets a list of all installed apps including appName,
  /// packageName, versionName, decodedIcon, installDate and lastUpdateDate.
  /// ```
  /// To view app icon set includeAppIcon to true and use Image.network(decodedIcon).
  Future<List<App>> getInstalledApps({
    bool includeSystemApps = false,
    bool onlyIncludeAppsWithLaunchIntent = true,
    bool includeAppIcon = false,
  }) async {
    try {
      List appsInfo = await _channel.invokeMethod("getInstalledApps", {
        "includeSystemApps": includeSystemApps,
        "onlyIncludeAppsWithLaunchIntent": onlyIncludeAppsWithLaunchIntent,
        "includeAppIcon": includeAppIcon,
      });
      List<App> apps = List<App>();
      for (Map appInfo in appsInfo) {
        apps.add(App.fromMap(appInfo));
      }
      return apps;
    } on PlatformException catch (e) {
      _throwError(e, "getting installed apps");
      return [];
    }
  }

  /// This method launches an app by its packageName on device and if it's not installed
  /// it will launch them on play store or the browser depending on if google play store
  /// is installed.
  /// ```
  /// Note that you can set forceAppStore to true if you want to force
  /// the app to open on google play store for example if you want users to rate your app.
  Future<String> launchApp(
    String packageName, {
    bool forceAppStore = false,
  }) async {
    assert(packageName.isNotEmpty, "packageName can not be empty");
    try {
      String launchResult = await _channel.invokeMethod(
        "launchApp",
        {"packageName": packageName, "forceAppStore": forceAppStore},
      );
      return launchResult;
    } on PlatformException catch (e) {
      _throwError(e, "opening $packageName");
      return "FailedToOpen";
    }
  }

  /// This method prompts a dialog to the user that asks for his confirmation
  /// to uninstall the specified app.
  Future<void> uninstallApp(String packageName) async {
    assert(packageName.isNotEmpty, "packageName can not be empty");
    try {
      await _channel.invokeMethod("uninstallApp", {"packageName": packageName});
    } on PlatformException catch (e) {
      _throwError(e, "uninstalling $packageName");
    }
  }

  /// This method creates "backups" folder if not exists under your app files folder
  /// and saves a backup apk of the specified app.
  /// ```
  /// Note that all backups gets stored in your app file directory
  /// under a sub directory called ""backups"
  /// for example if your app package name is com.example.name then backups will be
  /// stored under /storage/emulated/0/Android/data/com.example.name/files/backups .
  Future<String> makeBackup(String packageName) async {
    assert(packageName.isNotEmpty, "packageName can not be empty!");
    try {
      String backupResult = await _channel.invokeMethod(
        "makeBackup",
        {"packageName": packageName},
      );
      return backupResult;
    } on PlatformException catch (e) {
      _throwError(e, "making a backup for $packageName");
      return "UnexpectedError";
    }
  }

  /// This method gets a map of all stored backups in the "backups" folder under your app files
  /// represented as the packageName as key and the apkPath as value.
  Future<Map> getBackups() async {
    try {
      Map backups = await _channel.invokeMethod("getBackups");
      return backups;
    } on PlatformException catch (e) {
      _throwError(e, "sharing an app");
      return {};
    }
  }

  /// This method prompts a dialog to the user that asks for his confirmation
  /// to install an apk from the specified path.
  Future<String> installAppFromBackup(String apkPath) async {
    assert(apkPath.isNotEmpty, "apkPath can not be empty!");
    try {
      bool calledSuccessfully = await _channel.invokeMethod(
        "installAppFromBackup",
        {"apkPath": apkPath},
      );
      return calledSuccessfully ? "CalledSuccessfully" : "FileNotFound";
    } on PlatformException catch (e) {
      _throwError(e, "installing an app from backup");
      return "UnexpectedError";
    }
  }

  Future<String> shareAppViaBluetooth(String packageName) async {
    try {
      String shareResult = await _channel.invokeMethod(
        "shareAppViaBluetooth",
        {"packageName": packageName},
      );
      return shareResult;
    } on PlatformException catch (e) {
      _throwError(e, "sharing an app");
      return "UnexpectedError";
    }
  }

  static void _throwError(PlatformException e, String caller) {
    print("unexpected error was caught while $caller");
    print("error code: ${e.code}");
    print("error message: ${e.message}");
    print("error cause: ${e.details}");
  }
}
